{*************************************************************}
{            RegControls for Delphi 32                        }
{ Version:   1.0                                              }
{ Author:    Aleksey Kuznetsov                                }
{ E-Mail:    info@utilmind.com                                }
{ Home Page: www.utilmind.com                                 }
{ Created:   May, 16, 1999                                    }
{ Modified:  May, 17, 1999                                    }
{ Legal:     Copyright (c) 1999, UtilMind Solutions           }
{*************************************************************}
{ TRegForm - automaticly saves form placement and size to     }
{            SystemRegistry                                   }
{ TRegComboBox - saves history list to SystemRegistry         }
{*************************************************************}
{ PROPERTIES:                                                 }
{   RegEnabled: Boolean - Use Registry or not                 }
{   RegShowWarning: Boolean - if True then component will show}
{                         warnings on changing RegKey property}
{                         in design time. Experienced         }
{                         programers can always set this      }
{                         property to False                   }
{   RegLocation: TRegLocation - Current_User or Local_Machine }
{   RegKey: String - Key in System Registry for storing data. }
{                    Be extremely cautious by setting or      }
{                    changing this property                   }
{*************************************************************}
{ METHODS (for TRegComboBox only):                            }
{   Save - added current .Text to history and saves it to     }
{          SystemRegistry.                                    }
{*************************************************************}
{ Please see demo program for more information.               }
{*************************************************************}
{                     IMPORTANT NOTE:                         }
{ This software is provided 'as-is', without any express or   }
{ implied warranty. In no event will the author be held       }
{ liable for any damages arising from the use of this         }
{ software.                                                   }
{ Permission is granted to anyone to use this software for    }
{ any purpose, including commercial applications, and to      }
{ alter it and redistribute it freely, subject to the         }
{ following restrictions:                                     }
{ 1. The origin of this software must not be misrepresented,  }
{    you must not claim that you wrote the original software. }
{    If you use this software in a product, an acknowledgment }
{    in the product documentation would be appreciated but is }
{    not required.                                            }
{ 2. Altered source versions must be plainly marked as such,  }
{    and must not be misrepresented as being the original     }
{    software.                                                }
{ 3. This notice may not be removed or altered from any       }
{    source distribution.                                     }
{*************************************************************}

unit RegControls;

interface

uses
  Windows, Classes, Controls, StdCtrls, Forms, Registry;

type
  TRegLocation = (CurrentUser, LocalMachine);

{ ----- TRegForm ----- }

  TRegForm = class(TComponent)
  private
    ParentForm: TForm;
    OldCloseQuery: TCloseQueryEvent;
    FRegEnabled: Boolean;
    FRegShowWarning: Boolean;
    FRegLocation: TRegLocation;
    FRegKey: String;

    procedure SetRegLocation(Value: TRegLocation);
    procedure CloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure DoSave;
    procedure SetRegKey(Value: String);    
  protected
    procedure Loaded; override;
  public
    constructor Create(aOwner: TComponent); override;
    procedure Save;
  published
    property RegEnabled: Boolean read FRegEnabled write FRegEnabled;
    property RegShowWarning: Boolean read FRegShowWarning write FRegShowWarning;
    property RegLocation: TRegLocation read FRegLocation write SetRegLocation;
    property RegKey: String read FRegKey write SetRegKey;
  end;

{ ----- TRegComboBox ----- }

  TRegComboBox = class(TComboBox)
  private
    FRegEnabled: Boolean;
    FRegShowWarning: Boolean;
    FRegLocation: TRegLocation;
    FRegKey: String;
    FMaxCount: Integer;

    procedure SetRegLocation(Value: TRegLocation);
    procedure DoSave;
    procedure SetRegKey(Value: String);
  protected
    procedure Loaded; override;
  public
    constructor Create(aOwner: TComponent); override;
    procedure Save;
  published
    property RegEnabled: Boolean read FRegEnabled write FRegEnabled;
    property RegShowWarning: Boolean read FRegShowWarning write FRegShowWarning;
    property RegLocation: TRegLocation read FRegLocation write SetRegLocation;
    property RegKey: String read FRegKey write SetRegKey;
    property RegMaxCount: Integer read FMaxCount write FMaxCount;
  end;

procedure Register;

implementation

{ ----- TRegForm ----- }

var
  Reg: TRegistry;
  WS: TWindowState;

constructor TRegForm.Create(aOwner: TComponent);
begin
  inherited Create(aOwner);
  ParentForm := TForm(aOwner);
  OldCloseQuery := ParentForm.OnCloseQuery;
  ParentForm.OnCloseQuery := CloseQuery;
  FRegEnabled := True;
  FRegKey := '\Software\Default';
  FRegShowWarning := True;
end;

procedure TRegForm.Loaded;
begin
  inherited Loaded;
  if FRegEnabled and not (csDesigning in ComponentState) then
   try
     Reg := TRegistry.Create;
     try
       if FRegLocation = CurrentUser then
        Reg.RootKey := HKEY_CURRENT_USER
       else
        Reg.RootKey := HKEY_LOCAL_MACHINE;
       Reg.OpenKey(FRegKey, False);
       Reg.ReadBinaryData('RCWindowState', WS, SizeOf(WS));
       ParentForm.WindowState := WS;
       if WS <> wsMaximized then
        begin
         ParentForm.Left := Reg.ReadInteger('RCLeft');
         ParentForm.Top := Reg.ReadInteger('RCTop');
         ParentForm.Width := Reg.ReadInteger('RCWidth');
         ParentForm.Height := Reg.ReadInteger('RCHeight');
        end;
     except
     end;
   finally
     Reg.Free;
   end;
end;

procedure TRegForm.DoSave;
begin
  if ParentForm <> nil then
   begin
    Reg.WriteInteger('RCLeft', ParentForm.Left);
    Reg.WriteInteger('RCTop', ParentForm.Top);
    Reg.WriteInteger('RCWidth', ParentForm.Width);
    Reg.WriteInteger('RCHeight', ParentForm.Height);
    WS := ParentForm.WindowState;
    Reg.WriteBinaryData('RCWindowState', WS, SizeOf(WS));
   end;
end;

procedure TRegForm.Save;
begin
  if not (csDesigning in ComponentState) then
   begin
    try
      Reg := TRegistry.Create;
      try
       if FRegLocation = CurrentUser then
        Reg.RootKey := HKEY_CURRENT_USER
       else
        Reg.RootKey := HKEY_LOCAL_MACHINE;
       Reg.OpenKey(FRegKey, True);
       DoSave;
      except
      end;
    finally
      Reg.Free;
    end;
   end;  
end;

procedure TRegForm.CloseQuery(Sender: TObject; var CanClose: Boolean);
begin
  if Assigned(OldCloseQuery) then
   OldCloseQuery(Sender, CanClose);
  if FRegEnabled and CanClose and (ParentForm.Handle <> 0) then
   try
    Save;
   except
    Application.HandleException(Self);
   end;                                                 
end;

procedure TRegForm.SetRegKey(Value: String);
begin
  if (Value <> '') and (Value <> FRegKey) then
   begin
    if Value[Length(Value)] = '\' then
     SetLength(Value, Length(Value) - 1);
    FRegKey := Value;
    if FRegShowWarning and (csDesigning in ComponentState) and
       not (csReading in ComponentState) then
     Application.MessageBox('Please be exremely cautious by working with System Registry!'#13#10'Wrong setting of this property can DESTROY useful information'#13#10'about other programs!', 'Warning', mb_Ok or mb_IconExclamation);
   end;
end;

procedure TRegForm.SetRegLocation(Value: TRegLocation);
begin
  if FRegLocation <> Value then
   begin
    FRegLocation := Value;
    if not (csDesigning in ComponentState) and not (csReading in ComponentState) and
       not (csLoading in ComponentState) then
     begin
      try
       Reg := TRegistry.Create;
       try
         Reg.RootKey := HKEY_CURRENT_USER;
         try
           Reg.DeleteKey(FRegKey);
         except
         end;
         Reg.RootKey := HKEY_LOCAL_MACHINE;
         try
           Reg.DeleteKey(FRegKey);
         except
         end;
         if FRegLocation = CurrentUser then
          Reg.RootKey := HKEY_CURRENT_USER
         else
          Reg.RootKey := HKEY_LOCAL_MACHINE;
         Reg.OpenKey(FRegKey, True);
         try
           DoSave;
         except
         end;
       except
       end;
      finally
       Reg.Free;
      end;
     end;
   end;
end;

{ ----- TRegComboBox ----- }

constructor TRegComboBox.Create(aOwner: TComponent);
begin
  inherited Create(aOwner);
  FRegEnabled := True;
  FMaxCount := 20;
  FRegKey := '\Software\Default';
  FRegShowWarning := True;    
end;

procedure TRegComboBox.Loaded;
var
  St: TStringList;
begin
  inherited Loaded;
  if FRegEnabled and not (csDesigning in ComponentState) then
   try
     Reg := TRegistry.Create;
     try
       if FRegLocation = CurrentUser then
        Reg.RootKey := HKEY_CURRENT_USER
       else
        Reg.RootKey := HKEY_LOCAL_MACHINE;
       Reg.OpenKey(FRegKey, False);
       St := TStringList.Create;
       Reg.GetValueNames(St);
       Items.Assign(St);
       St.Destroy;
     except
     end;
   finally
     Reg.Free;
   end;
end;

procedure TRegComboBox.DoSave;
var
  i: Integer;
begin
  Reg.DeleteKey(FRegKey);
  Reg.OpenKey(FRegKey, True);
  i := Items.Count;
  if i > FMaxCount then i := FMaxCount;
  if i <> 0 then
   for i := 0 to i - 1 do
    Reg.WriteString(Items[i], '');
end;

procedure TRegComboBox.Save;
var
  i: Integer;
  b: Boolean;
begin
  if not (csDesigning in ComponentState) and (Text <> '') then
   begin
    repeat
     b := True;
     i := Items.Count;
     if i <> 0 then
      for i := 0 to i - 1 do
       if Items[i] = Text then
        begin
         Items.Delete(i);
         b := False;
        end;
    until b;
    Items.Insert(0, Text);
    try
      Reg := TRegistry.Create;
      try
        if FRegLocation = CurrentUser then
         Reg.RootKey := HKEY_CURRENT_USER
        else
         Reg.RootKey := HKEY_LOCAL_MACHINE;
        DoSave;
      except
      end;
    finally
      Reg.Free;
    end;
   end;
end;

procedure TRegComboBox.SetRegKey(Value: String);
begin
  if (Value <> '') and (Value <> FRegKey) then
   begin
    if Value[Length(Value)] = '\' then
     SetLength(Value, Length(Value) - 1);
    FRegKey := Value;
    if FRegShowWarning and (csDesigning in ComponentState) and
       not (csReading in ComponentState) then
     Application.MessageBox('Please be exremely cautious by working with System Registry!'#13#10'Wrong setting of this property can DESTROY useful information'#13#10'about other programs!', 'Warning', mb_Ok or mb_IconExclamation);
   end;
end;

procedure TRegComboBox.SetRegLocation(Value: TRegLocation);
begin
  if FRegLocation <> Value then
   begin
    FRegLocation := Value;
    if not (csDesigning in ComponentState) and not (csReading in ComponentState) and
       not (csLoading in ComponentState) then
     begin
      try
       Reg := TRegistry.Create;
       try
         Reg.RootKey := HKEY_CURRENT_USER;
         try
           Reg.DeleteKey(FRegKey);
         except
         end;
         Reg.RootKey := HKEY_LOCAL_MACHINE;
         try
           Reg.DeleteKey(FRegKey);
         except
         end;
         if FRegLocation = CurrentUser then
          Reg.RootKey := HKEY_CURRENT_USER
         else
          Reg.RootKey := HKEY_LOCAL_MACHINE;
         Reg.OpenKey(FRegKey, True);
         try
           DoSave;
         except
         end;
       except
       end;
      finally
       Reg.Free;
      end;
     end;
   end;
end;

procedure Register;
begin
  RegisterComponents('UtilMind', [TRegForm, TRegComboBox]);
end;

end.
